#パッケージのインストール＆読込
install.packages("quantmod")
library(quantmod)



#アップルの株価を取得。デフォルトのデータソースはYahoo
getSymbols("AAPL")



#取得結果は同名の変数(この例だとAAPL)に代入されている
print(AAPL)





#GEの株価を取得し、表示
getSymbols("GE")


print(GE)




#FREDからUSDJPYレートを取得
getSymbols("DEXJPUS",src="FRED")


print(DEXJPUS)




#2013年5末までの直近一年間のS&P500のデータ
sp500 <- getSymbols("^GSPC",
  src="yahoo",
  from=as.Date("2012-05-31"),
  to=as.Date("2013-05-31"),
  auto.assign = FALSE)


print(sp500)




#パッケージのインストールとライブラリのロード
install.packages("Quandl")
library(Quandl)



Quandl.auth("ここに認証トークン")



#日本の人口データの取得
japan.pop <- Quandl("WORLDBANK/JPN_SP_POP_TOTL")


print(japan.pop)



vix <- Quandl("YAHOO/INDEX_VIX",
              start_date="2005-12-31",
              end_date="2012-12-31",
              collapse="monthly",
              type="xts")



#時系列処理用のzooパッケージをロードし、そこに搭載されている描画関数を呼び出し
library(zoo)
plot.zoo(vix$Close,ylab=c("VIX"),lwd=3,main="VIX index",format="%Y/%m")



#パッケージのインストールと読込み
install.packages("TFX")
library(TFX)



QueryTrueFX()



options(digits.secs=3)



QueryTrueFX()



fx <- QueryTrueFX()
class(fx)



raw <- QueryTrueFX(parse.response=FALSE)
raw



ParseTrueFX(raw)



pair.specific <- ConnectTrueFX("USD/JPY,EUR/JPY", username="登録したユーザ名", password="登録したパスワード")


QueryTrueFX(pair.specific)



pair.specific <- ConnectTrueFX("", username="登録したユーザ名", password="登録したパスワード")
QueryTrueFX(pair.specific)



# csvフォーマットにてデータを取得するためのセッション作成
pair.specific <- ConnectTrueFX("", username="登録したユーザ名", password="登録したパスワード" ,format='csv')
# csv形式で為替データ取得
QueryTrueFX(pair.specific,parse=FALSE)



pair.specific <- ConnectTrueFX("", username="登録したユーザ名", password="登録したパスワード" )
# 10回連続で問い合わせを行い、取得データの行数を表示
for(i in 1:5){print(nrow(QueryTrueFX(pair.specific)))}



pair.specific <- ConnectTrueFX("", username="登録したユーザ名", password="登録したパスワード" , snapshot=TRUE)
#10回連続で問い合わせを行い、取得データ行数を表示
for(i in 1:5){print(nrow(QueryTrueFX(pair.specific)))}



TrueFXRef("web")



#データがあるURLの指定
URL <- "http://www.mof.go.jp/english/jgbs/reference/interest_rate/jgbcme.csv"
#直近の金利データの読込。stringsAsFactorは文字列型を強制的にファクター型へと変換させる
jgb.yield <- read.csv(URL, stringsAsFactor=FALSE)


print(jgb.yield)


#長期金利(10年)の変化を可視化する
#x軸の描画をしない(plot関数によるx軸描画の抑制)
par(xaxt="n")
#X10(10Y金利)を指定の線幅(lwd)・色(col)で描画する
plot(jgb.yield$X10,type="l",lwd=3, col=2, xlab="", ylab = "Rate")
#x軸の描画を再開
par(xaxt="s")
#x軸に日付(Date)を表示する
axis(1, 1:nrow(jgb.yield), jgb.yield$Date)



# JPX国債先物ボラティリティ・インデックスデータのURL
URL <- "http://www.ose.or.jp/f/jgbs/27/historical_datas/JGBF-VI_Histrical_Data.xls"
# ファイルのダウンロード
download.file(URL, "JGBF-VI.xls", mode="wb")


#XLConnectパッケージのインストールと読込み
install.packages("XLConnect")
library(XLConnect)



# ダウンロードしたファイルを読み込み
jgbfvi <- loadWorkbook("JGBF-VI.xls")


jgbfvi[1]


plot(jgbfvi[1], type="l")