price <- c(100,110,95,120)



# データの差分
diff(price)
# データの変化率
price[-1]/price[-length(price)]-1
# データの対数変化率
diff(log(price))



price[-1]/price[-length(price)]-1



# x=c(1,2,3), y=c(10,12,15)という2つのベクトルを用意
x <- 1:3
y <- c(10, 12, 15)
#ベクトルの要素毎の割り算を計算( c(10/1=10, 12/2=6, 15/3=5))
y/x



library(quantmod)
library(TTR)



#変化率
ROC(price, type="discrete")
#対数変化率
ROC(price, type="continuous")



#変化率(NAなし)
ROC(price, type="discrete", na.pad=FALSE)



# EuStockMarketsをデータフレーム型に変換したものをeu.stock.marketsとする
eu.stock.markets <- as.data.frame(EuStockMarkets)
head(eu.stock.markets)



# eu.stock.markets(EuStockMarkets)の全列の変化率を計算する
eu.stock.markets.return <- apply(eu.stock.markets, 2, ROC, na.pad=FALSE)


eu.stock.markets.return




library(quantmod)
# 2012年5月31日から2013年5月31日のGoogleの株価データを取得
google <- getSymbols("GOOGL",
  src="yahoo",
  from=as.Date("2012-05-31"),
  to=as.Date("2013-05-31"),
  auto.assign = FALSE)


# 取得したデータの表示
google


par(oma = c(0, 0, 0, 1))
chartSeries(google)


# Googleの株価データの終値の収益率を計算
google.close <- ROC(google[,4], type="discrete", na.pad=FALSE)
# リスク(収益率の標準偏差)を計算
sd(google.close)



# 年率化したリスク(収益率の標準偏差)を計算
sqrt(250)*sd(google.close)



# 欧州圏株価変化率に対する相関行列の計算
cor(eu.stock.markets.return)



# 1-10の値をもつサンプル
x <- 1:10



# xの平均(mean)と標準偏差(sd)を計算
mean(x)
sd(x)



#scale関数を用いてxを標準化
x.scaled <- scale(x)



# x.scaledの平均・標準偏差を計算
mean(x.scaled)
sd(x.scaled)



# 元のデータの平均(scaled:center)・標準偏差(scaled:scale)を取得
attr(x.scaled, "scaled:center")
attr(x.scaled, "scaled:scale")



# 適当なデータの作成
x <- matrix(1:50, ncol=5)


x


# 複数系列を持つデータの標準化
x.scaled <- apply(x, 2, scale)
# 全ての列が標準化されているかの確認
apply(x.scaled, 2, mean)
apply(x.scaled, 2, sd)



cars.xts <- xts(cars, order.by=as.Date("2013-11-01")+1:nrow(cars))


cars.xts


#サンプル価格データの作成
price <- data.frame(A=100+cumsum(rnorm(6)),B=200+cumsum(rnorm(6)))
#サンプル日付データの作成(2013年11月1日から2014年4月1日まで、一カ月毎)
sample.dates <- seq.Date(as.Date("2013-11-01"),as.Date("2014-4-01"), "months")
#日付をorder.by引数に付与し、priceをデータとしたxts型データへ変換
price <- xts(price, order.by=sample.dates)
#欠損作成
price[1,1] <- NA
price[2,1] <- NA
price[4,1] <- NA
price[1,2] <- NA
price[3,2] <- NA
price[4,2] <- NA
price[6,2] <- NA
#作成した欠損値入りのデータを表示
price



# 1から5の数からなるベクトルの累積和を計算
# 1 = 1
# 1 + 2 = 3
# 1 + 2 + 3 = 6 
# 1 + 2 + 3 + 4 = 10 
# 1 + 2 + 3 + 4 + 5 = 15
cumsum(1:5)



#欠損値を直近の値で補間
na.locf(price)



#欠損値を次の時点の値で補間
na.locf(price, fromLast=TRUE)



#欠損値を線形に補間
na.approx(price)



#欠損値をスプライン補間
na.spline(price)



#欠損値をナチュラル・スプライン補間
na.spline(price, method="natural")



#NAを0に置き換える
na.fill(price, 0)



#NAが少なくとも一方の列(A,B)に含まれている行(日)を削除
na.omit(price)



#少なくとも１つNAがある場合はFALSE、NAがない場合はTRUEとなるベクトルを取得
index.complete <- complete.cases(price)
#NAが少なくとも一方の列(A,B)に含まれている行(日)を削除(na.omitと同じ)
price[index.complete]
#na.omitで除去されたデータを抽出(!演算子を用いると論理値を反転させることが出来る)
price[!index.complete]



#デフォルトでは「各列の平均値」より、欠損値を置き換え
na.aggregate(price)



#欠損していない箇所を用いて"年"毎の平均値を計算＆その値による補間
na.aggregate(price, format, "%Y")



# 2013年7月7日の日付データ作成
sample.date <- as.Date("2013-07-07")
sample.date
# 日付を"年(下二桁)/月/日"(yy/mm/ddフォーマット)で表示
format(sample.date, "%y/%m/%d")
# 日付を"年(四桁)/月/日"(yyyy/mm/ddフォーマット)で表示
format(sample.date, "%Y/%m/%d")



format(sample.date, "%Y")



#データの日付部分。index関数はxts型の時系列データから日付部分のみを抽出する関数
as.yearmon(index(price))



months(index(price))



# 年の中央値で補間
na.aggregate(price, format, "%Y", FUN=median)



na.aggregate(price, format, "%Y", FUN=function(x)0.5*(mean(x)+median(x)))



#na.trim関数を全てデフォルトの設定で適用
na.trim(price)



# データの"両"端において(sides引数のデフォルトは"both")、
# "全列"のデータが欠損している場合("all")、除外
na.trim(price, is.na="all")
# データの"左"端(時間的に過去,"left")において、
# "全列"のデータが欠損している場合("all")、除外
na.trim(price, sides="left", is.na="all")
#データの"両"端("both")において、
# "いずれかの列"のデータが欠損している場合("any")、除外。デフォルトの設定はこれ。
na.trim(price, sides="both", is.na="any")



#TTRパッケージを読み込み(ROC関数を使用するため)
library(TTR)


#ユーロ市場の株式指数データからDAXのみ抽出
dax.index <- EuStockMarkets[,"DAX"]
#指数を変化率に変換し、データの先頭にある欠損値を削除
dax.return <- na.omit(ROC(dax.index, type="discrete"))



# hist関数を用い、分割数を30個と指定してヒストグラムを描画
hist(dax.return, breaks=30)



# DAX指数リターンの絶対値(abs関数)が「平均値(mean関数)+3*標準偏差(sd関数)超える」ケースを
# TRUEとして返却する論理値ベクトルを作成。TRUEの箇所がはずれ値に該当すると考える
outlier.index <- abs(dax.return) > mean(dax.return) + 3*sd(dax.return)



#はずれ値ではない(!outlier.index)データでヒストグラムを描画
hist(dax.return[!outlier.index])



#はずれ値と見做した値を取得
dax.return[outlier.index]



# 箱ひげ図を描画
boxplot(dax.return)



# 外れ値の定義を変更し、箱ひげ図を再描画
boxplot(dax.return, range = 3)



# boxplot関数が外れ値と見做した値を取得。
# plot引数をFALSEに設定することで描画が抑制される
outlier <- boxplot(dax.return, plot=FALSE)$out


outlier

# outliersパッケージのインストールと読込
install.packages("outliers")
library(outliers)



# SG関数を取りこむ
source("http://aoki2.si.gunma-u.ac.jp/R/src/SG.R", encoding="EUC-JP")



set.seed(100)


# 平均を0, 8とずらした正規分布からのサンプリング
x <- c(rnorm(30), rnorm(3, mean=8))



# ヒストグラムの表示
hist(x)



# outlierパッケージのgrubbs.test関数でSmirnov-Grubbs検定
x.grubbs <- grubbs.test(x)
x.grubbs



# 群馬大学・青木先生によるSmirnov-Grubbs検定の関数
x.sg <- SG(x)
x.sg



# サンプルデータベクトルx中の外れ値を取得
x.outlier <- outlier(x)
x.outlier



#外れ値以外を抽出
x[x != x.outlier]



# Smirnov-Grubbs検定を用いて、
# 指定した有意水準(p.threshold)でデータ(x)から外れ値を除去する関数
rm.outliers.by.grubbs <- function(x, p.threshold=0.05)
{
  repeat{
    # Smirnov-Grubbs検定を実行
    x.grubbs <- grubbs.test(x)
    # P値が有意水準(p.threshold)以下の場合、
    # 帰無仮説が棄却されたとして、外れ値を元のデータから除去、
    # そうでなければ、これ以上の外れ値はないとしてループ(repeat)から脱出
    if(x.grubbs$p.value < p.threshold){
      x.outlier <- outlier(x)
      x <- x[x != x.outlier]
    }else{
      break
    }
  }
  #外れ値を削除した後のデータを返り値として返却
  x
}



#有意水準5%でDAX指数のリターン系列から外れ値を除去
p.threshold <- 0.05


hist(rm.outliers.by.grubbs(dax.return, p.threshold))



# rnorm関数で平均:3%, 標準偏差:20%を持つ正規分布に従う乱数を生成し、疑似リターンデータとし、xへ代入
set.seed(10)
x <- rnorm(100, mean=0.03, sd=0.2)



#作成したデータ(x)を引数に指定、また検定したい分布の累積分布関数(pnorm)を指定
#指定した分布のパラメータも別途与える必要がある(正規分布なので平均と標準偏差を与えている)
ks.test(x,"pnorm",mean=mean(x),sd=sd(x))



# データxの平均値・標準偏差を計算
mean(x)
sd(x)



#作成したデータ(x)を引数に指定し、シャピロ-ウィルク検定を実行
shapiro.test(x)



#googleの株価リターン系列が正規分布しているかをコロモゴロフ・スミノフ検定にて確認
ks.test(google.close ,"pnorm",mean=mean(google.close),sd=sd(google.close))