# EuStockMarketsのデータ型を改めて確認
class(EuStockMarkets)



# EuStockMarketsの可視化
plot(EuStockMarkets)



# 100から始まる2系列からなる疑似株価データ(price)を作成
x <- cumprod(1+rnorm(10, sd=0.1))*100
y <- cumprod(1+rnorm(10, sd=0.1))*100
price <- cbind(x, y)
price



# 1から5の数からなるベクトルの累積積を計算
# 1 = 1
# 1 * 2 = 2
# 1 * 2 * 3 = 6
# 1 * 2 * 3 * 4 = 24
# 1 * 2 * 3 * 4 *5 = 120
x <- 1:5
cumprod(x)



# ts型の疑似データへと変換
price.ts <- ts(price, start = c(2013, 1), frequency = 12)
price.ts



# price.tsの描画
plot(price.ts)



# price.tsの描画(1パネル)
plot(price.ts, plot.type="single", lty=1:2, main="Single plot")



# 4半期のデータとして再定義
price.ts <- ts(price, start = c(2013, 1), frequency = 4)
price.ts



# GEのデータを取得し、data.geという変数に代入
data.ge <- getSymbols("GE", auto.assign=FALSE)
class(data.ge)


data.ge


# ラインチャートを描画
chartSeries(data.ge, type="line", subset="2009",TA=NULL)



# バーチャートを描画(データ期間の最初の4か月)
# バーの色：緑色=上昇、オレンジ色=下落
# バーの長さ：上端=高値、下端=安値
# バーの左右の突起：左側=始値、右側=終値
chartSeries(data.ge, type="bar", subset="first 4 months",TA=NULL)



# ローソク足チャートを描画(2012/11/01～2013/2/1までのデータ)
# ローソク足の色：緑色=上昇、オレンジ色=下落
chartSeries(data.ge, type="candlesticks", subset="2012-11-01::2013-02-01",TA=NULL)



# テーマをモノクロ白(white.mono)としたローソク足チャート
candleChart(data.ge, theme=chartTheme("white.mono"), subset="2012-11-01::2013-02-01",TA=NULL)



# 上昇時のロウソク足を黒塗り、下落時のロウソク足を白塗り」するようにテーマを変更
# (見やすいように背景色も白に変更(area, bg.col))
my.theme <- chartTheme(up.col="#0000FF",dn.col="white",area="white",bg.col="white")
candleChart(data.ge, theme=my.theme, subset="2012-11-01::2013-02-01",TA=NULL)



# 変更可能な項目の一覧を表示
chartTheme()



## # R言語で取得可能な色の名前一覧を文字列ベクトルとして出力
## colors()


print.omit(colors(), m=5)




# 取引高も追加したローソク足チャート例
candleChart(data.ge, subset="2012-01::2012-06")
# ↓も同様の動作
# candleChart(data.ge, subset="2012-01::2012-06", TA="addVo()")



# quatmodパッケージのHELPを開く
help(package=quantmod)



# ボリンジャーバンドとMACD指標を追加したチャートを表示
chartSeries(data.ge, subset="2012-01::2012-03", TA="addMACD();addBBands()")



# ヒストリカル・
historical.volatility <- function(ohlc){
  # Close価格データ(4列目に存在と仮定)をリターン系列へ変換
  return.close <- ROC(ohlc[,4])
  # 各日付におけるボラティリティを計算(デフォルトでは直近10個のデータで計算)
  runSD(return.close)
}



# GEのデータ系列に対してヒストリカル・ボラティリティを計算
historical.volatility(data.ge)


# newTA関数によりhistorical.volatilityを新しいTA指標とする
addHistoricalVolatility <- newTA(historical.volatility, col="red",lwd=3)
# addHistoricalVolatilityをTAとして追加し描画
chartSeries(data.ge, subset="2012-01::2012-03", TA="addHistoricalVolatility()")



# xts型(1系列)の描画(終値(Close)データ)
# major.format引数により、X軸の日付のフォーマットを指定できる
plot(data.ge[,4], major.format="%Y/%m")
# ↓のコードも同様の動作
# plot.xts(data.ge[,4], major.format="%Y/%m")



?strptime



# 現在のロケールを表示
Sys.getlocale("LC_TIME")



# 適当な日付型データを作成
x <- as.Date("2013-7-7")
# "曜日, 月名 日付　年"と表示
format(x, "%A, %B %d %Y")
# 現在のロケールを保存
locale <- Sys.getlocale("LC_TIME"); 
# C言語風な表示に切り替え
Sys.setlocale("LC_TIME", "C")
# "曜日, 月名 日付　年"と表示
format(x, "%A, %B %d %Y")
# ロケールを元に戻す
Sys.setlocale("LC_TIME", locale)



# data.ge変数の全ての変数を描画？
plot(data.ge)



# data.geのデータ型を確認
class(data.ge)



# zoo型のデータ描画関数plot.zooにより描画
plot.zoo(data.ge)



# xtsExtraをインストール
install.packages("xtsExtra", repos="http://R-Forge.R-project.org")



library(xtsExtra)



# xtsExtraパッケージによって上書きされたplot.xts関数により描画
plot(data.ge)



# 通常の描画 + "2008-2009年の背景色を灰色" + "2009年3月5日にMinimum pointという印"
plot(data.ge[,1], events=list(time="2009-03-05", label="Minimum point"), blocks=list(start.time="2008-01-01", end.time="2009-12-31", col="gray"))



# xtsExtraパッケージのplot.xts関数のHELPを表示
?xtsExtra::plot.xts



# PerformanceAnalyticsパッケージのインストールと読込
install.packages("PerformanceAnalytics")
library(PerformanceAnalytics)



# edhecデータの読込と表示
data(edhec)


edhec


# managersデータの読込と表示
data(managers)


managers


# 「Convertible Arbitrage, CTA Global, Distressed Securities」
# という取引戦略(1~3列目に対応)の累積パフォーマンスを表示。
edhec.sub <- edhec[,1:3]


chart.CumReturns(edhec.sub, main="Cumulative Returns", wealth.index=TRUE, legend.loc="topleft")



# edhecの累積リターンを計算
edhec.cumreturn <- apply(edhec.sub, 2, function(x)cumprod(1+x))


edhec.cumreturn


# 要約統計量の表示
summary(managers)



# 要約統計量の表示
table.Stats(managers)



# managersデータの全列に対する年率化されたパフォーマンスを表示
table.AnnualizedReturns(managers)



# HAM1-HAM3のデータの年率化されたリターン・リスクを表示
chart.RiskReturnScatter(managers[,1:3])



# table.Returns関数を用いて、
# HAM1ファンドマネージャー(1列目)とSP500(8列目)のパフォーマンスを比較
table.Returns(managers[,c(1, 8)])



# 棒グラフで2000年のリターンデータを可視化
chart.Bar(managers["2000", 1:2], legend.loc="topright", main="Bar chart")



# 棒グラフで2000年のリターンデータを可視化
charts.Bar(managers["2000", 1:2], legend.loc="topright", main="Bar chart")



# ラインチャートで可視化
chart.TimeSeries(edhec[,1])



# ドローダウンテーブルを作成
table.Drawdowns(managers[,2])



# HAM1とSP500のデータを取得
sp500 <- managers[,8]
ham1 <- managers[,1]
# 回帰分析を実行
lm(ham1~sp500)



# 回帰係数の取得
beta <- coef(lm(ham1~sp500))[2]



# システマティックリスクを計算
beta*sd(sp500)*sqrt(12)



# システマティックリスクを計算(パッケージの関数)
SystematicRisk(ham1, sp500)



# CAPM.beta関数とStdDev.annualized関数を用いてシステマティックリスクを計算
CAPM.beta(ham1, sp500) * StdDev.annualized(sp500)



# ham1とsp500の回帰分析の結果をregression変数として保持
regression <- lm(ham1~sp500)
# 回帰分析の結果の残差を取得
residual <- regression$residuals
# その"標本"分散を計算
sample.variance <- var(residual) * (length(residual)-1)/(length(residual))
# 標本分散からアンシステマティックリスクを計算
# sqrt(12)は年率化のファクタ
sqrt(sample.variance)*sqrt(12)



SpecificRisk(ham1, sp500)



# ベンチマークをsp500と設定し、HAM1~5までのリスクの分解を実施
table.SpecificRisk(managers[,1:5], sp500)



#Equity Market Neutral(株式ニュートラル戦略)のデータのヒストグラムをqqplot付で表示
chart.Histogram(edhec[,"Equity Market Neutral",drop=FALSE], methods = c("add.qqplot","add.risk") )



# HAM1, HAM2の散布図、およびSP500に対する回帰結果の描画
chart.Regression(managers[, 1:2, drop = FALSE],managers[, 8, drop = FALSE], fit = c("loess", "linear"), legend.loc = "topleft")



# 相関係数の計算、na.omit関数によって欠損値データは削除
cor(na.omit(managers[,1:8]))



# ダミーデータの作成と相関の計算
x <- seq(-5,5,0.01)
y <- (function(x){x^2+rnorm(length(x))})(x)
cor(x,y)



# ダミーデータの散布図を表示
plot(x,y)



#managersデータの1-5列目のデータ本体のみをcoredata関数で取得
x <- coredata(managers[,1:5])


#散布図を描画
pairs(x)



#HAM1とHAM2のみをHAM2 > 0の範囲で描画
pairs(~HAM1 + HAM2, data=x, subset=HAM2>0)



# chart.Correlation関数による描画
chart.Correlation(x)



# managersの1列目(HAM1)と2列目(HAM2)、managersの8列目(SP500.TR)と9列目(US.10Y.TR)の
# 全ての組に対してローリング相関係数を計算
x <- managers[, 1:2]
y <- managers[, 8:9]


chart.RollingCorrelation(x, y, legend.loc="bottomleft", width=12, main = "Rolling Correlation(12-Month)")