## 6.2
setwd("~/Documents/R/")
library(XLConnect)
library(xts)
library(quantmod)
library(forecast)

# Henry Hubのデータを取得
# http://www.eia.gov/dnav/ng/hist/rngwhhdM.htm
henry.hub <- readWorksheetFromFile("RNGWHHDm.xls", 2, startRow=3) 
henry.hub[,1] <- as.Date(henry.hub[,1]) # 日付型へ変換
henry.hub <- xts(x=henry.hub[,2], order.by=henry.hub[,1]) # xts型に変換

# グラフ
plot.zoo(henry.hub, main="Henry Hub Gulf Coast Natural Gas", 
         xlab="Date", ylab="") 

# コレログラムを作成
Acf(henry.hub, lag.max=50)

# Ljung-Box検定
Box.test(henry.hub, lag=50, type="Ljung-Box")

## 6.3
setwd("~/Documents/R/")
library(XLConnect)
library(xts)
library(quantmod)
library(forecast)

# 日経225のデータを取得
nikkei <- getSymbols("^N225", src="yahoo", from=as.Date("2003-04-01"), 
                     to=as.Date("2013-03-31"), auto.assign = FALSE)

nikkei.price <- nikkei$N225.Close

#収益率（対数収益率）
nikkei.return <- diff(log(nikkei.price))[-1]

plot.zoo(nikkei.return, main="NIKKEI 225 Return", xlab="Date", ylab="", ylim=c(-0.15, 0.15))

# コレログラムを作成
Acf(nikkei.return)

# Ljung-Box検定
Box.test(nikkei.return, lag=30, type="Ljung-Box")

# コレログラムを作成
Acf(nikkei.return**2)

# 収益率が正規分布に従うと仮定
n <- nrow(nikkei.return)

mu <- mean(nikkei.return)

sigma <- sqrt((n - 1)/n)*sd(nikkei.return)

set.seed(1)
normal.return <- rnorm(n=n, mean=mu, sd=sigma)

normal.return <- xts(normal.return, index(nikkei.return))

plot.zoo(normal.return, main="Normal Distrubuted Return", xlab="Date", ylab="", ylim=c(-0.15, 0.15))

library(MASS)
library(stats)

# t分布への当てはめ
t.param <- fitdistr(nikkei.return, densfun="t")$estimate

# t分布からのサンプル
set.seed(1)
t.return <- rt(n, df=t.param["df"]) * t.param["s"] + t.param["m"] 

t.return <- xts(t.return, index(nikkei.return))

plot.zoo(t.return, main="t Distrubuted Return", xlab="Date", ylab="", ylim=c(-0.15, 0.15))

# QQプロット
x <- sort(as.vector(coredata(normal.return)))
qqnorm(x, main="Normal Q-Q Plot of Normal Returns")

# 直線の追加
qqline(x)

# QQプロット
x <- sort(as.vector(coredata(t.return)))
qqnorm(x, main="Normal Q-Q Plot of t Returns")

# 直線の追加
qqline(x)

# QQプロット
x <- sort(as.vector(coredata(nikkei.return)))
qqnorm(x, main="Normal Q-Q Plot of Actual Returns")

# 直線の追加
qqline(x)


## 6.4
setwd("~/Documents/R/")
library(XLConnect)
library(xts)
library(quantmod)
library(forecast)


# MA(1), theta_1=0.9, sigma=1
x <- arima.sim(n=100, model=list(ma=0.9, sd=1))
plot(x, main="MA(1), theta_1=0.9, sigma=1")

# Acf
Acf(x, main="AutoCorrelations: MA(1), theta_1=0.9, sigma=1")

# MA(1), theta_1=-0.9, sigma=1
x <- arima.sim(n=100, model=list(ma=-0.9, sd=1))
plot(x, main="MA(1), theta_1=-0.9, sigma=1")

# MA(1), theta_1=0.9, sigma=2
x <- arima.sim(n=100, model=list(ma=0.9, sd=2))
plot(x, main="MA(1), theta_1=0.9, sigma=2")

# MA(2), theta_1=0.5, theta_2=0.4, sigma=1
x <- arima.sim(n=100, model=list(ma=c(0.7, 0.4), sd=1))
plot(x, main="MA(2), theta_1=0.7, theta_2=0.4, sigma=1")

# Acf
Acf(x, main="AutoCorrelations: MA(2), theta_1=0.7, theta_2=0.4, sigma=1")

# AR(1), phi_1=0.9, sigma=1
x <- arima.sim(n=100, model=list(ar=0.9, sd=1))
plot(x, main="AR(1), phi_1=0.9, sigma=1")

# Acf
Acf(x, main="AutoCorrelations: AR(1), phi_1=0.9, sigma=1")

# AR(1), phi_1=-0.9, sigma=1
x <- arima.sim(n=100, model=list(ar=-0.9, sd=1))
plot(x, main="AR(1), phi_1=-0.9, sigma=1")

# AR(2), phi_1=0.5, phi_2=-0.8, sigma=1
x <- arima.sim(n=100, model=list(ar=c(0.5, -0.8), sd=1))
plot(x, main="AR(2), phi_1=0.5, phi_2=-0.8, sigma=1")

# Acf
Acf(x, main="AutoCorrelations: AR(2), phi_1=0.5, phi_2=-0.8, sigma=1")

# シミュレーション回数
N <- 10000
# 100点の将来シミュレーションをN回繰り返して，行列として保存．
x <- sapply(1:N, function(i) arima.sim(n=100, model=list(ar=c(0.5, 0.4))))
# 期待値
mu <- apply(x, 1, mean)

# 両側50%区間
upper <- apply(x, 1, function(z) quantile(z, probs=0.75, type=1))
lower <- apply(x, 1, function(z) quantile(z, probs=0.25, type=1))

# matplotで複数系列をプロット
matplot(1:100, data.frame(mean=mu, upper=upper, lower=lower), type="l", ylab="", xlab="")

# Henry Hubのデータを取得
# http://www.eia.gov/dnav/ng/hist/rngwhhdM.htm
henry.hub <- readWorksheetFromFile("RNGWHHDm.xls", 2, startRow=3) 
henry.hub[,1] <- as.Date(henry.hub[,1]) # 日付型へ変換
henry.hub <- xts(x=henry.hub[,2], order.by=henry.hub[,1]) # xts型に変換

x <- auto.arima(x=henry.hub, d=0, D=0, max.p=5, max.q=5, stationary=T, seasonal=F, ic="aic", trace=F)
summary(x)

plot(forecast(x, level = c(50,95), h = 60),shadecols=c("yellow", "orange"),fcol=1)
abline(h=mean(henry.hub))


## 6.5
setwd("~/Documents/R/")
library(XLConnect)
library(xts)
library(quantmod)
library(forecast)

# 日経225のデータを取得
nikkei <- getSymbols("^N225", src="yahoo", from=as.Date("2003-04-01"), 
                     to=as.Date("2013-03-31"), auto.assign = FALSE)

nikkei.price <- nikkei$N225.Close

#収益率（対数収益率）
nikkei.return <- diff(log(nikkei.price))[-1]

library(tseries)
x <- garch(nikkei.return, order=c(1,1), trace=F) 
summary(x)


## 6.6
setwd("~/Documents/R/")
library(XLConnect)
library(xts)
library(quantmod)
library(forecast)

# 多通貨の為替レートに対するVARモデル

# varsパッケージをロードする
library(vars)

# 文字コードを指定してファイルポインタを作成
fp <- file("http://www.mizuhobank.co.jp/rate/market/csv/quote.csv", encoding="cp932")
# 全行をstring型のvectorとして読み込む
curncy.raw <- readLines(fp)
# ファイルポインタを閉じる
close(fp)

# 3行目をヘッダとして取っておく
curncy.header <- strsplit(curncy.raw[3], ",")[[1]]
curncy.header[1] <- "Date"
# 1〜3行目以外を，カンマで分割してリストとして保存
curncy.str <- sapply(curncy.raw[-c(1:3)], function(l) strsplit(l, ","))
# リストをstring型のmatrixとして保存
curncy <- do.call("rbind", curncy.str)
# 行名が残っているので削除
row.names(curncy) <- NULL
# *****はNAに変換
curncy[curncy == "*****"] <- NA

# 日付は変換して取っておく
curncy.date <- as.Date(curncy[,1])
# それ以外はdouble型に
curncy <- apply(curncy[,-1], 2, as.double)

# matrixをdataframeに変換．日付をマージ．
curncy <- as.data.frame(curncy)
curncy <- cbind(curncy.date, curncy)

# ヘッダを挿入．空列は削除．
colnames(curncy) <- curncy.header
curncy <- curncy[,colnames(curncy) != ""]

# お好みでxts型に変換
curncy <- xts(curncy[,-1], curncy[,1])

# 最初の5通貨を試しにプロット
plot.zoo(curncy[,1:5], main="5 currencies")

# 多通貨の為替レートに対するVARモデル

# 最初の3通貨に限定
curncy.selected <- curncy[,c(1:3)]
# 月末レートに限定
curncy.selected <- curncy.selected[endpoints(curncy, on="months")]
# 最新はカット
curncy.selected <- curncy.selected[-nrow(curncy.selected),]

# 対数差分（リターン）に変換
curncy.return <- diff(log(curncy.selected))[-1]
# VARの推定
x <- VAR(curncy.return)
summary(x)

causality(x,cause="USD") 

impulse <- irf(x,impulse="USD",response=c("GBP","EUR"),boot=FALSE)
plot(impulse)

## 6.7
setwd("~/Documents/R/")
library(XLConnect)
library(xts)
library(quantmod)
library(forecast)

# 日経225のデータを取得
nikkei <- getSymbols("^N225", src="yahoo", from=as.Date("2003-04-01"), 
                     to=as.Date("2013-03-31"), auto.assign = FALSE)

nikkei.price <- nikkei$N225.Close

#収益率（対数収益率）
nikkei.return <- diff(log(nikkei.price))[-1]

adf.test(nikkei.price)
adf.test(nikkei.return)